<?php

  /*

   Generic PHP Netflix Parsing Class

   -- Copyright --
   Parts Copyright 2004, by Paul Burdick (reedmaniac@pmachine.com)
   Parts Copyright 2004, by James McDuffie (ttq3sxl02@sneakemail.com)

   -- License --
   GNU GPL

   -- Description --

   Downloads a user's Netflix queue or out list from the Netflix website
   then caches and parses it.

   -- Example Usage --

    $netflix_id = 'PXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX';
    $netflix_cache_dir = '/tmp/netflix_cache';

    $nf_parse = new NetFlixParse($netflix_id, $netflix_cache_dir);
    $nf_parse->parse_data($which_list);
    $items = $nf_parse->get_items();

    foreach ($items as $curr_item) {
       print $curr_item['link'];
       ... etc ...
    }

    An item in the items array has the following valid keys:
      link             - URL to the current movie
      movie_id         - Netflix ID of the movie, can be used to get images
      title            - Name of the movie
      rating           - MPAA Rating
      genre            - Genre
      user_stars       - The star rating of the user
      predicted_stars  - The star rating from all Netflix users combined

   -- History --

   0.1 - Original version for Expression Engine by Paul Burdick
   0.2 - Modifications by James McDuffie:
      * Removed expression engine specifics
      * Made class more generic
      * Added parsing of star ratings

   Plugin based off class written by Paul Burdick for ExpressionEngine
   reedmaniac@pmachine.com
   http://plugins.pmachine.com

  */

define(NETFLIX_PARSE_VERSION, '0.2'); 

require_once('ExpiringCache.php');

Class NetFlixParse {

  // Object that handles caching
  var $cache_obj;

  // URL for retrieving Queue
  var $netflix_url   = 'http://www.netflix.com/Queue';

  // ID for NetFlix account
  var $netflix_id    = '';

  // Items Data
  var $items	     = array();

  // HTML from the netflix page
  var $netfix_page_data = '';

  //-------------------------------------
  //  Constructor
  //-------------------------------------

  function NetFlixParse($id, $root_cache_path, $cache_refresh = 1440) {
   	
    // -------------------------------
    //  Set Parameters 
    // -------------------------------
    	
    $this->netflix_id = $id;

    if ($this->netflix_id == '') {
      trigger_error("No Netflix ID specified");
    }
    	
    // -------------------------------
    // Setup cache object
    // -------------------------------

    $cache_base_filename = 
      $root_cache_path . '/' . md5($this->netflix_id) . "-nf_site_html";

    $this->cache_obj = new ExpiringCache($cache_base_filename, $cache_refresh);
					  
    // -------------------------------
    // Retrieve NetFlix Data
    // -------------------------------
        
    if ($this->cache_obj->generation_needed()) {
      $this->retrieve_data();
      $this->cache_obj->store_data_into_cache($this->netflix_page_data);
    } else {
      $this->netflix_page_data = $this->cache_obj->get_cache_data();
    }

  }
  // END
	
  //-------------------------------------
  //  Retreive NetFlix HTML File
  //-------------------------------------   

  function retrieve_data() {

    $cookies = "Cookie: NetflixShopperId={$this->netflix_id};\r\n";
			
    $target = parse_url($this->netflix_url);
			
    $fp = @fsockopen($target['host'], 80, $errno, $errstr, 15);
			
    if (! is_resource($fp))
      {
	return false;
      }
			
    // Send the request
    fputs ($fp,"GET " . $this->netflix_url . " HTTP/1.0\r\n" ); 
    fputs ($fp,"Host: " . $target['host'] . "\r\n" ); 
    fputs ($fp, $cookies);
    fputs ($fp,"User-Agent: NetflixCheck/1.0\r\n\r\n");
			
    $getting_headers = true;
			
    while ( ! feof($fp))
      {
	$line = fgets($fp, 4096);
			
	if ($getting_headers == false)
	  {
	    $this->netflix_page_data .= $line;
	  }
	elseif (trim($line) == '')
	  {
	    $getting_headers = false;
	  }
      }
	
    @fclose($fp);

    // It is necessary to remove all returns, newlines and tabs for the
    // parsing to work succesfully
    $this->netflix_page_data =& 
      preg_replace("/(\r\n)|(\r)|(\n)|(\t)/", ' ', $this->netflix_page_data);
    	
    return true;
  }
  // END

  // --------------------------------
  // --------------------------------

  function get_items() {
    return $this->items;
  }
	
  // --------------------------------
  //  Parse Data
  // --------------------------------

  function parse_data($which='queue') {
    $tag = ($which == 'queue') ? 'bd' : 'or';
  	
    if (preg_match_all("/\<tr class\=".$tag."\>(.+?)\<\/tr\>/", $this->netflix_page_data, $matches)) {
	for($i=0; $i < sizeof($matches['0']); $i++)
	  {
	    if (preg_match_all("/\<td.+?\>(.+?)\<\/td\>/", $matches['1'][$i], $matches2))
	      {
		for($t=1; $t < sizeof($matches2['1']); $t++)
		  {
		    switch($t)
		      {
		      case '1':
			// Link, title, and item_id									
			if (preg_match("/\<a href\=\"((.+?)movieid=(.+?))\"\>(.+?)\<\/a\>/", $matches2['1'][$t], $matches3))
			  {
			    $this->items[$i+1]['link']		=  $matches3['1'];
			    $this->items[$i+1]['movie_id'] 	=& $matches3['3'];
			    $this->items[$i+1]['title']		=  $matches3['4'];								
			  }
			else
			  {
			    continue;
			  }
			break;
		      case '2':
			// Rating/MPAA
			$this->items[$i+1]['rating'] = $matches2['1'][$t];
			break;
		      case '3':
			// Genre
			$this->items[$i+1]['genre'] = $matches2['1'][$t];
			break;
		      case '4':
			if (preg_match("/StarbarInsert\((.*)\)\;/", $matches2['1'][$t], $starbar_options) ) {
			  list( $itemId, $starType, $numStars,
				$predictedRating, $isNotInterested, 
				$isNoOpinion,
				$showNI, $showNiMini, $showClear, 
				$showNoOpinion,$refreshWhenChanged ) = split(",", $starbar_options['1']);
			  $this->items[$i+1]['user_stars'] = $numStars;
			  $this->items[$i+1]['predicted_stars'] = $predictedRating;
			}
			break;
		      }
		  }
	      }			
	  }			
      }

  }
  // END

} // END NetFlixParse class
?>